=== added directory 'lib/lp/app/longpoll/javascript'
=== added file 'lib/lp/app/longpoll/javascript/longpoll.js'
--- lib/lp/app/longpoll/javascript/longpoll.js	1970-01-01 00:00:00 +0000
+++ lib/lp/app/longpoll/javascript/longpoll.js	2011-07-07 09:59:11 +0000
@@ -0,0 +1,197 @@
+/* Copyright 2009 Canonical Ltd.  This software is licensed under the
+ * GNU Affero General Public License version 3 (see the file LICENSE).
+ *
+ * The Launchpad Longpoll module provides the functionnality to deal
+ * with longpolling on the JavaScript side.
+ *
+ * The module method setupLongPollManager is called in every template and
+ * the long poll machinery will only be started if LP.cache.longpoll
+ * is populated.
+ *
+ * Usually the only thing you will want to do to use the long polling feature
+ * is to make sure LP.cache.longpoll is populated and then create Javascript
+ * handlers for the events which will be fired.
+ * You also might create handlers for the generic long polling events
+ * (see below).
+ * @module longpoll
+ */
+YUI.add('lp.app.longpoll', function(Y) {
+
+var namespace = Y.namespace('lp.app.longpoll');
+
+// Event fired when the long polling request starts.
+namespace.longpoll_start_event = 'lp.app.longpoll.start';
+
+// Event fired each time the long polling request fails (to connect or
+// to parse the returned result).
+namespace.longpoll_fail_event = 'lp.app.longpoll.failure';
+
+// Event fired when the delay between each failed connection is set to
+// a long delay (after MAX_SHORT_DELAY_FAILED_ATTEMPTS failed attempts).
+namespace.longpoll_longdelay = 'lp.app.longpoll.longdelay';
+
+// Event fired when the delay between each failed connection is set back
+// to a short delay.
+namespace.longpoll_shortdelay = 'lp.app.longpoll.shortdelay';
+
+namespace._manager = null;
+
+// After MAX_SHORT_DELAY_FAILED_ATTEMPTS failed connections (real failed
+// connections or connection getting an invalid return) separated
+// by SHORT_DELAY (millisec), wait LONG_DELAY (millisec) between
+// each failed connection.
+namespace.MAX_SHORT_DELAY_FAILED_ATTEMPTS = 5;
+namespace.SHORT_DELAY = 1000;
+namespace.LONG_DELAY = 3*60*1000;
+
+/**
+ *
+ * A Long Poll Manager creates and manages a long polling connexion
+ * to the server to fetch events. This class is not directly used
+ * but managed through 'setupLongPollManager' which creates and
+ * initialises a singleton LongPollManager.
+ *
+ * @class LongPollManager
+ */
+function LongPollManager(config) {
+    LongPollManager.superclass.constructor.apply(this, arguments);
+}
+
+LongPollManager.NAME = "longPollManager";
+
+Y.extend(LongPollManager, Y.Base, {
+    initializer : function(cfg) {
+        this._started = false;
+        this._failed_attempts = 0;
+        this._repoll = true;
+        this._sequence = 0;
+    },
+
+    setConnectionInfos : function(key, uri) {
+        this.key = key;
+        this.uri = uri;
+    },
+
+    _io : function (uri, config) {
+        Y.io(uri, config);
+    },
+
+    successPoll : function (id, response) {
+        try {
+            var data = Y.JSON.parse(response.responseText);
+            var event_key = data.event_key;
+            var event_data = data.event_data;
+            Y.fire(event_key, event_data);
+            return true;
+        }
+        catch (e) {
+            Y.fire(namespace.longpoll_fail_event, e);
+            return false;
+        }
+    },
+
+    failurePoll : function () {
+        Y.fire(namespace.longpoll_fail_event);
+    },
+
+    /**
+     * Return the delay (milliseconds) to wait before trying to reconnect
+     * again after a failed connection.
+     *
+     * The rationale here is that:
+     * 1. We should not try to reconnect instantaneously after a failed
+     *     connection.
+     * 2. After a certain number of failed connections, we should set the
+     *     delay between two failed connection to a bigger number because
+     *     the server may be having problems.
+     *
+     * @method _pollDelay
+     */
+    _pollDelay : function() {
+        this._failed_attempts = this._failed_attempts + 1;
+        if (this._failed_attempts >=
+                namespace.MAX_SHORT_DELAY_FAILED_ATTEMPTS) {
+            Y.fire(namespace.longpoll_longdelay);
+            return namespace.LONG_DELAY;
+        }
+        else {
+            return namespace.SHORT_DELAY;
+        }
+    },
+
+    /**
+     * Relaunch a connection to the server after a successful or
+     * a failed connection.
+     *
+     * @method repoll
+     * @param {Boolean} failed: whether or not the previous connection
+     *     has failed.
+     */
+    repoll : function(failed) {
+        if (!failed) {
+            if (this._failed_attempts >=
+                    namespace.MAX_SHORT_DELAY_FAILED_ATTEMPTS) {
+                Y.fire(namespace.longpoll_shortdelay);
+            }
+            this._failed_attempts = 0;
+            if (this._repoll) {
+                this.poll();
+            }
+        }
+        else {
+            var delay = this._pollDelay();
+            if (this._repoll) {
+                Y.later(delay, this, this.poll);
+            }
+        }
+    },
+
+    poll : function() {
+        var that = this;
+        var config = {
+            method: "GET",
+            sync: false,
+            on: {
+                failure: function() {
+                    that.failurePoll();
+                    that.repoll(true);
+                },
+                success: function(id, response) {
+                    var res = that.successPoll(id, response);
+                    that.repoll(res);
+                }
+            }
+        };
+        this._sequence = this._sequence + 1;
+        var queue_uri = this.uri +
+            "?uuid=" + this.key +
+            "&sequence=" + this._sequence;
+        if (!this._started) {
+            Y.fire(namespace.longpoll_start_event);
+            this._started = true;
+        }
+        this._io(queue_uri, config);
+    }
+});
+
+namespace.LongPollManager = LongPollManager;
+
+namespace.getLongPollManager = function() {
+    if (!Y.Lang.isValue(namespace._manager)) {
+        namespace._manager = new namespace.LongPollManager();
+    }
+    return namespace._manager;
+};
+
+namespace.setupLongPollManager = function() {
+    if (Y.Lang.isValue(LP.cache.longpoll)) {
+        var key = LP.cache.longpoll.key;
+        var uri = LP.cache.longpoll.uri;
+        var longpollmanager = namespace.getLongPollManager();
+        longpollmanager.setConnectionInfos(key, uri);
+        longpollmanager.poll();
+        return longpollmanager;
+    }
+};
+
+}, "0.1", {"requires":["base", "event", "lang", "json", "io"]});

=== added directory 'lib/lp/app/longpoll/javascript/tests'
=== added file 'lib/lp/app/longpoll/javascript/tests/test_longpoll.html'
--- lib/lp/app/longpoll/javascript/tests/test_longpoll.html	1970-01-01 00:00:00 +0000
+++ lib/lp/app/longpoll/javascript/tests/test_longpoll.html	2011-07-07 09:59:11 +0000
@@ -0,0 +1,30 @@
+<!DOCTYPE
+   HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
+   "http://www.w3.org/TR/html4/strict.dtd">
+<html>
+  <head>
+    <title>Launchpad Longpoll</title>
+    <!-- YUI 3.0 Setup -->
+    <script type="text/javascript"
+            src="../../../../../canonical/launchpad/icing/yui/yui/yui.js"></script>
+    <script type="text/javascript"
+            src="../../../../../canonical/launchpad/icing/lazr/build/lazr.js"></script>
+    <link rel="stylesheet"
+          href="../../../../../canonical/launchpad/icing/yui/cssreset/reset.css"/>
+    <link rel="stylesheet"
+          href="../../../../../canonical/launchpad/icing/yui/cssfonts/fonts.css"/>
+    <link rel="stylesheet"
+          href="../../../../../canonical/launchpad/icing/yui/cssbase/base.css"/>
+    <link rel="stylesheet"
+          href="../../../../../canonical/launchpad/javascript/test.css" />
+    <!-- Required modules -->
+    <script type="text/javascript" src="../../../../app/javascript/client.js"></script>
+    <!-- The module under test -->
+    <script type="text/javascript" src="../longpoll.js"></script>
+    <!-- The test suite -->
+    <script type="text/javascript" src="test_longpoll.js"></script>
+  </head>
+  <body class="yui3-skin-sam">
+    <div id="log"></div>
+  </body>
+</html>

=== added file 'lib/lp/app/longpoll/javascript/tests/test_longpoll.js'
--- lib/lp/app/longpoll/javascript/tests/test_longpoll.js	1970-01-01 00:00:00 +0000
+++ lib/lp/app/longpoll/javascript/tests/test_longpoll.js	2011-07-07 09:59:11 +0000
@@ -0,0 +1,277 @@
+/* Copyright (c) 2011, Canonical Ltd. All rights reserved. */
+
+YUI({
+    base: '../../../../../canonical/launchpad/icing/yui/',
+    filter: 'raw', combine: false, fetchCSS: false
+    }).use(
+        'test', 'console', 'node-event-simulate', 'json',
+        'lp.app.longpoll',
+        function(Y) {
+
+    var suite = new Y.Test.Suite("longpoll Tests");
+    var longpoll = Y.lp.app.longpoll;
+
+    var testLongPollSingleton = {
+        name: 'TestLongPollSingleton',
+        tearDown: function() {
+            // Cleanup the singleton;
+            longpoll._manager = null;
+        },
+
+        testGetSingletonLongPollManager: function() {
+            Y.Assert.isNull(longpoll._manager);
+            var manager = longpoll.getLongPollManager();
+            Y.Assert.isNotNull(longpoll._manager);
+            var manager2 = longpoll.getLongPollManager();
+            Y.Assert.areSame(manager, manager2);
+        },
+
+        testInitLongPollManagerNoLongPoll: function() {
+            // if LP.cache.longpoll.key is undefined: no longpoll manager
+            // is created by setupLongPollManager.
+            window.LP = {
+                links: {},
+                cache: {}
+            };
+
+            longpoll.setupLongPollManager(true);
+            Y.Assert.isNull(longpoll._manager);
+        },
+
+        testInitLongPollManagerLongPoll: function() {
+            window.LP = {
+                links: {},
+                cache: {
+                    longpoll: {
+                        key: 'key',
+                        uri: '/+longpoll/'
+                    }
+                }
+            };
+
+            longpoll.setupLongPollManager(true);
+            Y.Assert.isNotNull(longpoll._manager);
+        }
+    };
+
+    suite.add(new Y.Test.Case(testLongPollSingleton));
+
+    var testLongPoll = {
+        name: 'TestLongPoll',
+
+        setUp: function() {
+            var manager = longpoll.getLongPollManager();
+            manager._repoll = false;
+            this.createBaseLP();
+        },
+
+        tearDown: function() {
+            // Cleanup the singleton;
+            longpoll._manager = null;
+        },
+
+        createBaseLP:function() {
+            window.LP = {
+                links: {},
+                cache: {}
+            };
+        },
+
+        setupLPCache: function() {
+            LP.cache.longpoll = {
+                key: 'key',
+                uri: '/+longpoll/'
+            };
+        },
+
+       setupLongPoll: function(nb_calls) {
+            this.setupLPCache();
+            return longpoll.setupLongPollManager(true);
+        },
+
+        testInitLongPollManagerQueueName: function() {
+            var manager = this.setupLongPoll();
+            Y.Assert.areEqual(LP.cache.longpoll.key, manager.key);
+            Y.Assert.areEqual(LP.cache.longpoll.uri, manager.uri);
+            Y.Assert.isFalse(Y.Lang.isValue(manager.nb_calls));
+        },
+
+        testPollStarted: function() {
+            var fired = false;
+            Y.on(longpoll.longpoll_start_event, function() {
+                fired = true;
+            });
+            var manager = this.setupLongPoll();
+            Y.Assert.isTrue(fired, "Start event not fired.");
+        },
+
+        testPollFailure: function() {
+            var fired = false;
+            Y.on(longpoll.longpoll_fail_event, function() {
+                fired = true;
+            });
+            // Monkeypatch io to simulate failure.
+            var manager = longpoll.getLongPollManager();
+            manager._io = function(uri, config) {
+                config.on.failure();
+            };
+            this.setupLongPoll();
+            Y.Assert.isTrue(fired, "Failure event not fired.");
+        },
+
+        testSuccessPollInvalidData: function() {
+            var manager = longpoll.getLongPollManager();
+            var custom_response = "{{";
+            var response = {
+                responseText: custom_response
+            };
+            var res = manager.successPoll("2", response);
+            Y.Assert.isFalse(res);
+        },
+
+        testSuccessPollMalformedData: function() {
+            var manager = longpoll.getLongPollManager();
+            var response = {
+                responseText: '{ "event_data": "6" }'
+            };
+            var res = manager.successPoll("2", response);
+            Y.Assert.isFalse(res);
+         },
+
+         testSuccessPollWellformedData: function() {
+            var manager = longpoll.getLongPollManager();
+            var response = {
+                responseText: '{ "event_key": "4", "event_data": "6"}'
+            };
+            var res = manager.successPoll("2", response);
+            Y.Assert.isTrue(res);
+        },
+
+        testPollDelay: function() {
+            // Create event listeners.
+            var longdelay_event_fired = false;
+            Y.on(longpoll.longpoll_longdelay, function(data) {
+                longdelay_event_fired = true;
+            });
+            var shortdelay_event_fired = false;
+            Y.on(longpoll.longpoll_shortdelay, function(data) {
+                shortdelay_event_fired = true;
+            });
+            var manager = longpoll.getLongPollManager();
+            // Monkeypatch io to simulate failure.
+            manager._io = function(uri, config) {
+                config.on.failure();
+            };
+            Y.Assert.areEqual(0, manager._failed_attempts);
+            this.setupLongPoll();
+            Y.Assert.areEqual(1, manager._failed_attempts);
+            var i, delay;
+            for (i=0; i<longpoll.MAX_SHORT_DELAY_FAILED_ATTEMPTS-2; i++) {
+                Y.Assert.areEqual(i+1, manager._failed_attempts);
+                delay = manager._pollDelay();
+                Y.Assert.areEqual(delay, longpoll.SHORT_DELAY);
+            }
+            // After MAX_SHORT_DELAY_FAILED_ATTEMPTS failed attempts, the
+            // delay returned by _pollDelay is LONG_DELAY and
+            // longpoll_longdelay is fired.
+            Y.Assert.isFalse(longdelay_event_fired);
+            delay = manager._pollDelay();
+            Y.Assert.isTrue(longdelay_event_fired);
+            Y.Assert.areEqual(delay, longpoll.LONG_DELAY);
+
+            // Monkeypatch io to simulate success.
+            manager._io = function(uri, config) {
+                config.on.success();
+            };
+            // After a success, longpoll.longpoll_shortdelay is fired.
+            Y.Assert.isFalse(shortdelay_event_fired);
+            delay = manager.poll();
+            Y.Assert.isTrue(shortdelay_event_fired);
+           },
+
+        testPollUriSequence: function() {
+            // Each new polling increses the sequence parameter:
+            // /+longpoll/?uuid=key&sequence=1
+            // /+longpoll/?uuid=key&sequence=2
+            // /+longpoll/?uuid=key&sequence=3
+            // ..
+            var count = 0;
+            // Monkeypatch io to simulate failure.
+            var manager = longpoll.getLongPollManager();
+            manager._io = function(uri, config) {
+                Y.Assert.areEqual(
+                    '/+longpoll/?uuid=key&sequence=' + (count+1),
+                    uri);
+                count = count + 1;
+                var response = {
+                   responseText: '{"i":2}'
+                };
+                config.on.success(2, response);
+            };
+            this.setupLongPoll();
+            Y.Assert.isTrue(count === 1, "Uri not requested.");
+        },
+
+        testPollPayLoadBad: function() {
+            // If a non valid response is returned, longpoll_fail_event
+            // is fired.
+            var fired = false;
+            Y.on(longpoll.longpoll_fail_event, function() {
+                fired = true;
+            });
+            var manager = longpoll.getLongPollManager();
+            // Monkeypatch io.
+            manager._io = function(uri, config) {
+                var response = {
+                   responseText: "{non valid json"
+                };
+                config.on.success(2, response);
+            };
+            this.setupLongPoll();
+            Y.Assert.isTrue(fired, "Failure event not fired.");
+        },
+
+        testPollPayLoadOk: function() {
+            // Create a valid message.
+            var custom_event = 'my-event';
+            var custom_payload = {5: 'i'};
+            var custom_response = {
+                'event_key': custom_event,
+                'event_data': custom_payload
+            };
+            var fired = false;
+            Y.on(custom_event, function(data) {
+                fired = true;
+                Y.Assert.areEqual(data, custom_payload);
+            });
+            var manager = longpoll.getLongPollManager();
+            // Monkeypatch io.
+            manager._io = function(uri, config) {
+                var response = {
+                   responseText: Y.JSON.stringify(custom_response)
+                };
+                config.on.success(2, response);
+            };
+            this.setupLongPoll();
+            Y.Assert.isTrue(fired, "Custom event not fired.");
+        }
+
+
+    };
+
+    suite.add(new Y.Test.Case(testLongPoll));
+
+    // Lock, stock, and two smoking barrels.
+    var handle_complete = function(data) {
+        window.status = '::::' + JSON.stringify(data);
+        };
+    Y.Test.Runner.on('complete', handle_complete);
+    Y.Test.Runner.add(suite);
+
+    var console = new Y.Console({newestOnTop: false});
+    console.render('#log');
+
+    Y.on('domready', function() {
+        Y.Test.Runner.run();
+        });
+});

=== modified file 'lib/lp/app/templates/base-layout-macros.pt'
--- lib/lp/app/templates/base-layout-macros.pt	2011-06-07 20:33:10 +0000
+++ lib/lp/app/templates/base-layout-macros.pt	2011-07-05 16:55:01 +0000
@@ -107,14 +107,17 @@
   <metal:load-lavascript use-macro="context/@@+base-layout-macros/load-javascript" />
 
   <script id="base-layout-load-scripts" type="text/javascript">
-    LPS.use('node', 'event-delegate', 'lp', 'lp.app.links', function(Y) {
+    LPS.use('node', 'event-delegate', 'lp', 'lp.app.links', 'lp.app.longpoll', function(Y) {
         Y.on('load', function(e) {
             sortables_init();
             initInlineHelp();
             Y.lp.activate_collapsibles();
             activateFoldables();
             activateConstrainBugExpiration();
-            Y.lp.app.links.check_valid_lp_links();
+	    Y.lp.app.links.check_valid_lp_links();
+            // Longpolling will only start if
+	    // LP.cache.longpoll is populated.
+            Y.lp.app.longpoll.setupLongPollManager();
         }, window);
 
         // Hook up the function that dismisses the help window if we click

